package unitTest;

import static org.junit.jupiter.api.Assertions.assertThrows;
import static org.junit.jupiter.api.Assertions.assertTrue;

import java.text.ParseException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.DynamicTest;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.TestFactory;
import org.junit.jupiter.api.function.Executable;
import org.junit.jupiter.params.ParameterizedTest;
import org.junit.jupiter.params.provider.ValueSource;

import exceptions.CategoryInvalidException;
import exceptions.DateInvalidException;
import exceptions.DescriptionInvalidException;
import exceptions.ImageInvalidException;
import exceptions.ItemNonExistentException;
import exceptions.PriceInvalidException;
import exceptions.TitleInvalidException;
import project.Auction;
import project.Category;
import project.Item;
import project.ItemPicture;
import project.Seller;
import project.User;

public class SellerUnitTest {

	@BeforeEach
	public void reset() {

		Item.items = new ArrayList<Item>();
		User.users = new ArrayList<User>();
		Auction.auctions = new ArrayList<Auction>();
		Category.categories = new ArrayList<Category>();

	}

	@ParameterizedTest
	@ValueSource(ints = { 0, -1, 3, 5 })
	public void createAuctionError(int itemId)
			throws PriceInvalidException, DateInvalidException, ItemNonExistentException {

		Seller s = Seller.getInstance();
		assertThrows(ItemNonExistentException.class,
				() -> s.createAuction(Item.getItem(itemId), "22/10/2019 11:20:13", "23/10/2019 11:20:13", 10, 5));

	}

	@ParameterizedTest
	@ValueSource(ints = { 0, -1, 3, 5 })
	public void createAuctionWorking(int itemId)
			throws PriceInvalidException, DateInvalidException, ItemNonExistentException, ParseException {

		Item i = new Item();
		i.setId(itemId);

		Seller s = Seller.getInstance();
		assertTrue(s.createAuction(Item.getItem(itemId), "22/10/2019 11:20:13", "23/10/2019 11:20:13", 10, 5));

	}

	@TestFactory
	public Collection<DynamicTest> createAuctionAcceptanceTest() {

		ArrayList<Integer> items = new ArrayList<Integer>(Arrays.asList(0, 1, 1, 1, 1, 1, 1, 1));
		ArrayList<String> startTimes = new ArrayList<String>(
				Arrays.asList("10/11/2019 11:20:12", "11/11/2019 11:20:12", "111112019", "10/11/2019", "111112019",
						"10/11/2019 11:20:12", "10/11/2019 11:20:12", "10/11/2019 11:20:12"));
		ArrayList<String> endTimes = new ArrayList<String>(
				Arrays.asList("11/11/2019 11:20:12", "10/11/2019 11:20:12", "10112019", "111112019", "11/11/2019",
						"11/11/2019 11:20:12", "11/11/2019 11:20:12", "11/11/2019 11:20:12"));
		ArrayList<Integer> prices = new ArrayList<Integer>(Arrays.asList(20, 20, 20, 20, 20, -1, 0, 20, 20));
		ArrayList<Integer> bidIncrements = new ArrayList<Integer>(Arrays.asList(5, 5, 5, 5, 5, 5, 5, 0, -1));
		ArrayList<String> testNames = new ArrayList<String>(Arrays.asList("item not existent", "end before start",
				"both dates wrong format", "end wrong format", "start wrong format", "negative price",
				"price eqaul to zero", "increment equal to zero", "increment negative"));
		ArrayList<Class<? extends Exception>> exceptions = new ArrayList<Class<? extends Exception>>(
				Arrays.asList(ItemNonExistentException.class, DateInvalidException.class, DateInvalidException.class,
						DateInvalidException.class, DateInvalidException.class, PriceInvalidException.class,
						PriceInvalidException.class, PriceInvalidException.class, PriceInvalidException.class));
		Collection<DynamicTest> dynamicTests = new ArrayList<DynamicTest>();

		Seller s = Seller.getInstance();
		Item item = Item.getInstance();
		item.setId(1);

		for (int i = 0; i < items.size(); i++) {

			final int j = i;

			Executable exec = () -> assertThrows(exceptions.get(j), () -> s.createAuction(Item.getItem(items.get(j)),
					startTimes.get(j), endTimes.get(j), prices.get(j), bidIncrements.get(j)));
			DynamicTest dTest = DynamicTest.dynamicTest(testNames.get(i), exec);

			dynamicTests.add(dTest);

		}

		return dynamicTests;

	}

	@TestFactory
	public Collection<DynamicTest> createItemAcceptanceTest() {

		ArrayList<String> titles = new ArrayList<String>(
				Arrays.asList("", "9999", "t-#\\", "aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa", "title",
						"title", "title", "title", "title", "title"));
		ArrayList<String> descriptions = new ArrayList<String>(Arrays.asList("description", "description",
				"description", "description", "", "9999", "too long", "description", "description", "description"));
		ArrayList<String> categories = new ArrayList<String>(Arrays.asList("Sport and Outdoor", "Sport and Outdoor",
				"Sport and Outdoor", "Sport and Outdoor", "Sport and Outdoor", "Sport and Outdoor", "Sport and Outdoor",
				"Sport and Outdoor", "Sport and Outdoor", ""));
		ArrayList<String> testNames = new ArrayList<String>(Arrays.asList("title empty", "numeric title",
				"symbols in the title", "title too long", "description empty", "numeric description",
				"description too long", "no pictures inserted", "too many pictures", "category not existent"));
		ArrayList<Class<? extends Exception>> exceptions = new ArrayList<Class<? extends Exception>>(
				Arrays.asList(TitleInvalidException.class, TitleInvalidException.class, TitleInvalidException.class,
						TitleInvalidException.class, DescriptionInvalidException.class,
						DescriptionInvalidException.class, DescriptionInvalidException.class,
						ImageInvalidException.class, ImageInvalidException.class, CategoryInvalidException.class));

		Collection<DynamicTest> dynamicTests = new ArrayList<DynamicTest>();

		Seller s = Seller.getInstance();

		new Category("Sport and Outdoor");

		ItemPicture ip1 = new ItemPicture();
		ItemPicture ip2 = new ItemPicture();
		ip1.setId(1);
		ip2.setId(2);

		for (int i = 0; i < titles.size(); i++) {

			String prepareDescription = descriptions.get(i);
			int[] prepareImages = { 1, 2 };

			if (i == 6) {

				for (int k = 0; k < 600; k++) {

					prepareDescription += "a";

				}

			}

			if (i == 7) {

				prepareImages = new int[0];

			}

			if (i == 8) {

				prepareImages = new int[11];

				for (int l = 0; l < 11; l++) {

					ItemPicture ipn = new ItemPicture();
					ipn.setId(l + 10);

					prepareImages[l] = l + 10;

				}

			}

			final String description = prepareDescription;
			final int j = i;
			final int[] images = prepareImages;

			Executable exec = () -> assertThrows(exceptions.get(j),
					() -> s.createItem(titles.get(j), description, images, categories.get(j)));
			DynamicTest dTest = DynamicTest.dynamicTest(testNames.get(i), exec);

			dynamicTests.add(dTest);

		}

		return dynamicTests;

	}

	@Test
	public void createItemWorking()
			throws CategoryInvalidException, TitleInvalidException, DescriptionInvalidException, ImageInvalidException {

		Seller s = Seller.getInstance();

		ItemPicture ip1 = new ItemPicture();
		ItemPicture ip2 = new ItemPicture();
		ip1.setId(1);
		ip2.setId(2);

		new Category("Sport and Outdoor");

		int[] images = { 1, 2 };

		assertTrue(s.createItem("Red Bike", "Beautiful red bike, contact me", images, "Sport and Outdoor"));

	}

}
